
// The code below uses require.js, a module system for javscript:
// http://requirejs.org/docs/api.html#define
//
// You don't have to use require.js, and you can delete all of this if
// you aren't (make sure to uncomment the script tags in index.html also)


// Set the path to jQuery, which will fall back to the local version
// if google is down
require.config({
  baseUrl: 'js/lib',
  paths: {
    'jquery':
            ['//ajax.googleapis.com/ajax/libs/jquery/1.7.2/jquery.min',
             'jquery']
  }
});

var global = this;

require(['jquery', 'crafty'], function ($) {

  // ------------------------------------------------------------------------
  // Initialization
  // ------------------------------------------------------------------------

  // Global namespace of the game, containing configuration
  // and some global variables
  var SNAKE = {
    cell: {
      width: 15, // in pixels
      height: 15 // in pixels
    },
    map: {
      width: 30, // in cells
      height: 30 // in cells
    },
    snake: null, // the current snake entity
    points: 0, // number of points in the current game
  }

  // Some short names for configurable values
  var WIDTH = SNAKE.cell.width;
  var HEIGHT = SNAKE.cell.height;
  var STAGE_WIDTH = WIDTH * SNAKE.map.width;
  var STAGE_HEIGHT = HEIGHT * SNAKE.map.height;

  // Initiate Crafty and create the canvas context
  Crafty.init(STAGE_WIDTH, STAGE_HEIGHT);
  Crafty.canvas.init();

  // ------------------------------------------------------------------------
  // Components
  // ------------------------------------------------------------------------

  // Create Components that we are going to use in the game
  Crafty.c('SnakeCell', {
    init: function () {
      this.requires('2D, Canvas, Color, Collision')
          .color('blue')
          .attr({ w: WIDTH, h: HEIGHT })
          .collision()
          .onHit('SnakeCell', function (c) {
            gameOver();
          });
      return this;
    }
  });

  Crafty.c('Snake', {
    init: function () {
      this.positions = [];
      this.growNext = false;
      return this;
    },

    walk: function (grow) {
      this.positions.push(Crafty.e('SnakeCell').attr({ x: this.x, y: this.y }));
      if (!this.growNext && !grow) {
        var old = this.positions.shift();
        old.destroy();
      } else if (this.growNext && grow) {
        this.growNext = true;
      } else {
        this.growNext = false;
      }
      return this;
    },

    grow: function () {
      this.growNext = true;
      return this;
    }
  });

  Crafty.c('Apple', {
    init: function () {
      this.requires('2D, Canvas, Color, Collision')
          .color('green')
          .attr({ w: WIDTH, h: HEIGHT })
          .collision()
          .onHit('SnakeCell', function (c) {
            bonus();
            this.destroy();
          });
      return this;
    }
  });

  // ------------------------------------------------------------------------
  // Functions
  // ------------------------------------------------------------------------

  // Create functions used in various places
  var bonus = function () {
    SNAKE.snake.grow();
    SNAKE.points++;
    updatePoints();
  };

  var updatePoints = function () {
    $('#points').text(SNAKE.points);
  }

  var gameOver = function () {
    console.log('Game Over!');
    Crafty.scene('game');
  };

  var randomInt = function (min, max) {
    return Math.floor(Math.random() * (max - min + 1)) + min;
  };

  // ------------------------------------------------------------------------
  // Scenes
  // ------------------------------------------------------------------------

  // Create a scene containing the menu of the game, used to change options
  // or launch the game.
  Crafty.scene('menu', function () {
    Crafty.background('#ccc');
    Crafty.e('2D, DOM, Text')
          .attr({ w: 100, h: 20, x: 150, y: 120 })
          .text('<a href="#" onclick="Crafty.scene(\'game\'); return false;">Play</a>')
          .css({ 'text-align': 'center' });
  });

  // Create a scene for the actual game, contain the state of a game and its
  // algorithm.
  Crafty.scene('game', function () {
    Crafty.background('#ddd');
    SNAKE.points = 0;
    updatePoints();

    var speed = 5; // Number of frames per second

    // Game objects
    SNAKE.snake = Crafty.e('Snake, 2D, Keyboard')
          .attr({
            x: (SNAKE.map.width / 2) * WIDTH,
            y: (SNAKE.map.height / 2) * HEIGHT,
            dX: 1,
            dY: 0
          })
          .bind('KeyDown', function () {
            if (this.isDown('UP_ARROW') && this.dY !== 1) {
              this.dX = 0;
              this.dY = -1;
            } else if (this.isDown('DOWN_ARROW') && this.dY !== -1) {
              this.dX = 0;
              this.dY = 1;
            } else if (this.isDown('LEFT_ARROW') && this.dX !== 1) {
              this.dX = -1;
              this.dY = 0;
            } else if (this.isDown('RIGHT_ARROW') && this.dX !== -1) {
              this.dX = 1;
              this.dY = 0;
            }
          })
          .bind('EnterFrame', function () {
            // We want to slow down the snake to only a few frames per second
            if (Crafty.frame() % (60 / speed) !== 0) {
              return;
            }

            // Now, the snake will grow randomly
            var grow = false;
            if (Math.random() > 0.7) {
              grow = true;
            }

            // Let's move the snake, given its direction
            if (this.dX === 1) {
              this.x += WIDTH;
            } else if (this.dX === -1) {
              this.x -= WIDTH;
            } else if (this.dY === 1) {
              this.y += HEIGHT;
            } else if (this.dY === -1) {
              this.y -= HEIGHT;
            }

            if (this.x >= STAGE_WIDTH) {
              this.x -= STAGE_WIDTH;
            }
            if (this.x < 0) {
              this.x += STAGE_WIDTH;
            }
            if (this.y >= STAGE_HEIGHT) {
              this.y -= STAGE_HEIGHT;
            }
            if (this.y < 0) {
              this.y += STAGE_HEIGHT;
            }

            this.walk(grow);
          })
          .walk(true);

    SNAKE.appleEvent = Crafty.bind('EnterFrame', function () {
      if (randomInt(0, 300) === 0) {
        Crafty.e('Apple')
              .attr({
                x: randomInt(0, SNAKE.map.width) * WIDTH,
                y: randomInt(0, SNAKE.map.height) * HEIGHT
              });
      }
    });
  }, function () {
    Crafty.unbind('EnterFrame', SNAKE.appleEvent);
  });

  // ------------------------------------------------------------------------
  // Start
  // ------------------------------------------------------------------------

  // Start the menu scene
  Crafty.scene('menu');


  // ------------------------------------------------------------------------
  // We're done! :)
  // ------------------------------------------------------------------------
});

// END REQUIRE.JS CODE
// Remove all of this if not using require.js
